/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: obtaindatatypea.c,v 1.3 2007/08/31 09:01:32 piru Exp $

    Desc:
    Lang: English
*/

#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/utility.h>
#include <utility/tagitem.h>
#include <dos/dos.h>
#include <datatypes/datatypesclass.h>
#include <libraries/iffparse.h>
#include "datatypes_intern.h"
#include <stddef.h>

#undef DEBUG
#define DEBUG 0
#include <aros/debug.h>

/*****************************************************************************

    NAME */

	AROS_LH3(struct DataType *, ObtainDataTypeA,

/*  SYNOPSIS */
	AROS_LHA(ULONG           , type, D0),
	AROS_LHA(APTR            , handle, A0),
	AROS_LHA(struct TagItem *, attrs, A1),

/*  LOCATION */
	struct Library *, DataTypesBase, 6, DataTypes)

/*  FUNCTION

    Examine the data pointed to by 'handle'.

    INPUTS

    type    --  type of 'handle'
    handle  --  handle to examine
		o DTST_FILE, 'handle' should be a BPTR lock
		o DTST_CLIPBOARD, 'handle' should be a struct IFFHandle *).
		o DTST_RAM, 'handle' should be a STRPTR (DataType name).
		o DTST_MEMORY, 'handle' is ignored. DTA_SourceAddress and
		  DTA_SourceSize specify the data buffer.
    attrs   --  additional attributes (currently only for DTST_MEMORY).

    RESULT

    A pointer to a DataType or NULL if failure. IoErr() gives more information
    in the latter case:

    ERROR_NO_FREE_STORE     --  Not enough memory available
    ERROR_OBJECT_NOT_FOUND  --  Unable to open the data type object
    ERROR_NOT_IMPLEMENTED   --  Unknown handle type

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO

    ReleaseDataType()

    INTERNALS

    HISTORY

*****************************************************************************/
{
    AROS_LIBFUNC_INIT

    struct CompoundDatatype *cdt = NULL;
    struct CompoundDatatype *CurrentDT;
    ULONG GroupID;

    D(bug("%s:\n",__func__));

    GroupID = (ULONG)GetTagData(DTA_GroupID, 0, attrs);
    CurrentDT = (struct CompoundDatatype *)GetTagData(DTA_DataType, NULL, attrs);

    D(bug("%s: DTList 0x%lx\n",__func__,GPB(DataTypesBase)->dtb_DTList));
    ObtainSemaphoreShared(&(GPB(DataTypesBase)->dtb_DTList->dtl_Lock));

    switch(type)
    {
    case DTST_FILE:
	{
	    struct FileInfoBlock *fib;

    	    D(bug("datatypes.library/ObtainDataType: SourceType = DTST_FILE\n"));

	    if((fib = AllocDosObject(DOS_FIB, TAG_DONE)) != NULL)
	    {
		D(bug("datatypes.library/ObtainDataType: allocated DOS_FIB. Now calling ExamineLock\n"));

		cdt = ExamineLock(DataTypesBase, CurrentDT, GroupID, (BPTR)handle, fib);

		D(bug("datatypes.library/ObtainDataType: DTST_FILE. ExamineLock call returned\n"));

		FreeDosObject(DOS_FIB, fib);
	    }
	}
	break;

    case DTST_CLIPBOARD:
	{
	    struct ClipboardHandle *cbh;
	    UBYTE                   CheckArray[64 + 1];

   	    D(bug("datatypes.library/ObtainDataType: SourceType = DTST_CLIPBOARD\n"));

	    cbh = (struct ClipboardHandle *)((struct IFFHandle *)handle)->iff_Stream;

	    /* cbh->cbh_Req.io_ClipID = 0; NONONONONO!!!! io_ClipID was set up by reads in OpenIFF!! */
	    cbh->cbh_Req.io_Error = 0;
	    cbh->cbh_Req.io_Offset = 0;
	    cbh->cbh_Req.io_Command = CMD_READ;
	    cbh->cbh_Req.io_Data = CheckArray;
	    cbh->cbh_Req.io_Length = sizeof(CheckArray) - 1;

	    if(DoIO((struct IORequest*)&cbh->cbh_Req))
	    {
		SetIoErr(ERROR_OBJECT_NOT_FOUND);
	    }
	    else
	    {
		cbh->cbh_Req.io_Error = 0;
		cbh->cbh_Req.io_Offset = 0;

		if((LONG)cbh->cbh_Req.io_Actual < 12)
		    SetIoErr(ERROR_OBJECT_NOT_FOUND);
		else
		{
		    struct DTHookContext dthc;

		    CheckArray[(LONG) cbh->cbh_Req.io_Actual] = '\0';

		    dthc.dthc_SysBase = (struct Library *)SysBase;
		    dthc.dthc_DOSBase = DOSBase;
		    dthc.dthc_IFFParseBase = IFFParseBase;
		    dthc.dthc_UtilityBase = UtilityBase;
		    dthc.dthc_Lock = NULL;
		    dthc.dthc_FIB = NULL;
		    dthc.dthc_FileHandle = NULL;
		    dthc.dthc_IFF = (struct IFFHandle *)handle;
		    dthc.dthc_Buffer = CheckArray;
		    dthc.dthc_BufferLength = cbh->cbh_Req.io_Actual;

		    D(bug("datatypes.library/ObtainDataType: DTST_CLIPBOARD: Calling ExamineData\n"));

		    cdt = ExamineData(DataTypesBase,
				      &dthc,
				      CurrentDT,
				      GroupID,
				      CheckArray,
				      (UWORD)cbh->cbh_Req.io_Actual,
				      "",
				      NULL);

		    D(bug("datatypes.library/ObtainDataType: DTST_CLIPBOARD: ExamineData call returned\n"));
		}
	    }
	}
	break;

    case DTST_RAM:
	{
		struct Node *curnode;
		struct CompoundDatatype *cur;

		D(bug("datatypes.library/ObtainDataType: SourceType = DTST_RAM\n"));


		D(bug("datatypes.library/ObtainDataType: find descriptor with the name <%s>\n",
			handle));

		/*
		 * I use the sorted list here, which is connected through Node2
		 */

		if (CurrentDT)
		{
			curnode = CurrentDT->DT.dtn_Node2.ln_Succ;
		}
		else
		{
			curnode = getDTLIST->dtl_SortedList.lh_Head;
		}

		for(;
		    curnode && curnode->ln_Succ;
		    curnode = curnode->ln_Succ)
		{
			cur = (struct CompoundDatatype *) ((UBYTE*) curnode - offsetof(struct CompoundDatatype,DT.dtn_Node2));
			D(bug("%s: current CompoundDataType 0x%lx\n",__func__,cur));
			D(bug("groupid  = %c%c%c%c\n",
				 cur->DTH.dth_GroupID >> 24,
				 cur->DTH.dth_GroupID >> 16,
				 cur->DTH.dth_GroupID >> 8,
				 cur->DTH.dth_GroupID));
			D(bug("id       = %c%c%c%c\n",
				 cur->DTH.dth_ID >> 24,
				 cur->DTH.dth_ID >> 16,
				 cur->DTH.dth_ID >> 8,
				 cur->DTH.dth_ID));
			D(bug("flags    = %x\n",	 cur->DTH.dth_Flags));
			D(bug("pri      = %d\n",	 cur->DTH.dth_Priority));
			D(bug("name     = %s\n",	 cur->DTH.dth_Name));
			D(bug("basename = %s\n",	 cur->DTH.dth_BaseName));
			D(bug("pattern  = %s\n",	 cur->DTH.dth_Pattern));
			D(bug("masklen  = %d\n",	 cur->DTH.dth_MaskLen));

			if (GroupID && cur->DTH.dth_GroupID != GroupID) continue;


			if (handle)
			{
#if 0
				if (cur->DTH.dth_Flags & DTF_CASE)
				{
					if (Strcmp(cur->DTH.dth_Name,
							handle)==0)
					{
						D(bug("datatypes.library/ObtainDataType: case match\n"));
						cdt = cur;
						break;
					}
				}
				else
#endif
				{
					if (Stricmp(cur->DTH.dth_Name,
							handle)==0)
					{
						D(bug("datatypes.library/ObtainDataType: nocase match\n"));
						cdt = cur;
						break;
					}
				}
			}
			else
			{
				D(bug("datatypes.library/ObtainDataType: always match\n"));
				cdt = cur;
				break;
			}
		}
	}
	break;

    case DTST_MEMORY:
	{
	    APTR SourceAddress;
	    ULONG SourceSize;

    	    D(bug("datatypes.library/ObtainDataType: SourceType = DTST_MEMORY\n"));

	    SourceAddress = (APTR) GetTagData(DTA_SourceAddress, 0, attrs);
	    SourceSize    = GetTagData(DTA_SourceSize, 0, attrs);

	    if(SourceAddress == NULL || SourceSize == 0)
		SetIoErr(ERROR_OBJECT_NOT_FOUND);
	    else
	    {
		D(bug("datatypes.library/ObtainDataType: Valid args. Now calling ExamineMemory\n"));

		cdt = ExamineMemory(DataTypesBase, CurrentDT, GroupID, SourceAddress, SourceSize);

		D(bug("datatypes.library/ObtainDataType: DTST_FILE. ExamineMemory call returned\n"));
	    }
	}
	break;

    default:
	SetIoErr(ERROR_NOT_IMPLEMENTED);
	break;
    }

    if(cdt)
    {
	Forbid();

	cdt->OpenCount++;

	Permit();
    }

    ReleaseSemaphore(&(GPB(DataTypesBase)->dtb_DTList->dtl_Lock));

    if(IoErr() == ERROR_OBJECT_NOT_FOUND)
	SetIoErr(DTERROR_COULDNT_OPEN);

    D(bug("datatypes.library/ObtainDataType: Done. Returning %x\n", cdt));

    return (struct DataType *)cdt;

    AROS_LIBFUNC_EXIT
} /* ObtainDataTypeA */
